@@ -11,7 +11,6 @@
 #include "core/renderbackend.h"
 
 #include <QObject>
-#include <QRegion>
 
 struct wl_resource;
 
@@ -33,6 +32,13 @@ class SubSurfaceInterface;
 class SurfaceInterfacePrivate;
 class Transaction;
 class SyncReleasePoint;
+class RawSurfaceAttachedState;
+class RawSurfaceExtension;
+
+enum class ColorDescriptionType {
+    Normal,
+    Windows,
+};
 
 /**
  * The SurfaceRole class represents a role assigned to a wayland surface.
@@ -80,19 +86,18 @@ private:
 class KWIN_EXPORT SurfaceInterface : public QObject
 {
     Q_OBJECT
-    /**
-     * The opaque region for a translucent buffer.
-     */
-    Q_PROPERTY(QRegion opaque READ opaque NOTIFY opaqueChanged)
-    /**
-     * The current input region.
-     */
-    Q_PROPERTY(QRegion input READ input NOTIFY inputChanged)
-    Q_PROPERTY(QSizeF size READ size NOTIFY sizeChanged)
+
 public:
     explicit SurfaceInterface(CompositorInterface *compositor, wl_resource *resource);
     ~SurfaceInterface() override;
 
+    /**
+     * Returns @c true if the surface is being destroyed; otherwise returns @c false.
+     *
+     * The surface will be marked as tearing down after the aboutToBeDestroyed() signal is emitted.
+     */
+    bool tearingDown() const;
+
     /**
      * Returns the object id for this Wayland surface.
      */
@@ -129,12 +134,13 @@ public:
     void frameRendered(quint32 msec);
     bool hasFrameCallbacks() const;
 
-    std::unique_ptr<PresentationFeedback> takePresentationFeedback(Output *output);
+    std::shared_ptr<PresentationFeedback> presentationFeedback(LogicalOutput *output);
+    bool hasPresentationFeedback() const;
 
-    QRegion opaque() const;
-    QRegion input() const;
-    QRegion bufferDamage() const;
-    QRectF bufferSourceBox() const;
+    Region opaque() const;
+    Region input() const;
+    Region bufferDamage() const;
+    RectF bufferSourceBox() const;
     /**
      * Returns the buffer transform that had been applied to the buffer to compensate for
      * output rotation.
@@ -160,13 +166,7 @@ public:
      *
      * QPoint(0, 0) corresponds to the upper left corner of this surface.
      */
-    QRectF boundingRect() const;
-    /**
-     * Returns the size of the attached buffer, in device pixels.
-     *
-     * If no buffer is attached to this surface, an invalid QSize will be returned.
-     */
-    QSize bufferSize() const;
+    RectF boundingRect() const;
 
     /**
      * @returns The SubSurface for this Surface in case there is one.
@@ -271,7 +271,7 @@ public:
     LockedPointerV1Interface *lockedPointer() const;
 
     /**
-     * @returns Whether this SurfaceInterface wants idle to be inhibited on the Output it is shown
+     * @returns Whether this SurfaceInterface wants idle to be inhibited on the LogicalOutput it is shown
      * @see inhibitsIdleChanged
      */
     bool inhibitsIdle() const;
@@ -290,22 +290,18 @@ public:
      * @returns The SurfaceInterface for the @p native resource.
      */
     static SurfaceInterface *get(wl_resource *native);
-    /**
-     * @returns The SurfaceInterface with given @p id for @p client, if it exists, otherwise @c nullptr.
-     */
-    static SurfaceInterface *get(quint32 id, const ClientConnection *client);
 
     /**
      * @see ClientConnection::setScaleOverride
      */
     qreal scaleOverride() const;
     /**
-     * Convert a co-ordinate from kwin logical space to surface logical space
+     * Convert a coordinate from kwin logical space to surface logical space
      * @internal
      */
     QPoint toSurfaceLocal(const QPoint &point) const;
     /**
-     * Convert a co-ordinate from kwin logical space to surface logical space
+     * Convert a coordinate from kwin logical space to surface logical space
      * @internal
      */
     QPointF toSurfaceLocal(const QPointF &point) const;
@@ -341,10 +337,11 @@ public:
     Transaction *lastTransaction() const;
     void setLastTransaction(Transaction *transaction);
 
-    const ColorDescription &colorDescription() const;
+    ColorDescriptionType colorDescriptionType() const;
+    const std::shared_ptr<ColorDescription> &colorDescription() const;
     RenderingIntent renderingIntent() const;
 
-    void setPreferredColorDescription(const ColorDescription &descr);
+    void setPreferredColorDescription(const std::shared_ptr<ColorDescription> &descr);
 
     double alphaMultiplier() const;
 
@@ -366,6 +363,31 @@ public:
      */
     SurfaceInterface *mainSurface();
 
+    /**
+     * Should be called immediately after compositing a non-tearing frame
+     * but always at a minimum rate that guarantees forward progress for the application
+     * (for example 30Hz)
+     */
+    void clearFifoBarrier();
+    bool hasFifoBarrier() const;
+
+    /**
+     * Registers the specified @a extension. Returns the pending state for the extension.
+     *
+     * @internal
+     */
+    RawSurfaceAttachedState *addExtension(RawSurfaceExtension *extension);
+
+    /**
+     * Unregisters the specified @a extension. The state owned by the extension will be removed
+     * from the pending and all cached surface states.
+     *
+     * @internal
+     */
+    void removeExtension(RawSurfaceExtension *extension);
+
+    QPointF mapToMainSurface(const QPointF &localPoint) const;
+
 Q_SIGNALS:
     /**
      * This signal is emitted when the underlying wl_surface resource is about to be freed.
@@ -385,18 +407,15 @@ Q_SIGNALS:
      * @see buffer
      * @see damage
      */
-    void damaged(const QRegion &);
-    void opaqueChanged(const QRegion &);
-    void inputChanged(const QRegion &);
+    void damaged(const Region &);
+    void opaqueChanged(const Region &);
+    void inputChanged(const Region &);
     /**
      * This signal is emitted when the buffer transform has changed.
      */
     void bufferTransformChanged(KWin::OutputTransform);
     void bufferSourceBoxChanged();
-    /**
-     * This signal is emitted when the size of the attached buffer has changed.
-     */
-    void bufferSizeChanged();
+    void bufferChanged();
     /**
      * Emitted when the Surface becomes visible, i.e. a non-null buffer has been attached.
      */
@@ -418,7 +437,7 @@ Q_SIGNALS:
      */
     void childSubSurfaceAdded(SubSurfaceInterface *subSurface);
     /**
-     * Emitted whenver the child sub-surface @p subSurface is removed.
+     * Emitted whenever the child sub-surface @p subSurface is removed.
      */
     void childSubSurfaceRemoved(SubSurfaceInterface *subSurface);
     /**
@@ -456,61 +475,110 @@ Q_SIGNALS:
      */
     void committed();
 
-    /**
-     * This signal is emitted when a surface commit with the specified \a serial has been cached
-     * to be applied later.
-     */
-    void stateStashed(quint32 serial);
-
-    /**
-     * This signal is emitted when the state in a surface commit with the specified \a serial
-     * has been applied.
-     */
-    void stateApplied(quint32 serial);
-
 private:
     std::unique_ptr<SurfaceInterfacePrivate> d;
     friend class SurfaceInterfacePrivate;
 };
 
 /**
- * The SurfaceExtension class is the base class for wl_surface extensions. The SurfaceExtension
- * helps with managing extension state and keeping it in sync with the surface state.
+ * The RawSurfaceAttachedState type represents the state of a surface extension.
  */
-template<typename Commit>
-class SurfaceExtension : public QObject
+class RawSurfaceAttachedState
+{
+public:
+    virtual ~RawSurfaceAttachedState()
+    {
+    }
+
+    /**
+     * Moves the data from this state into the @a target state.
+     */
+    virtual void mergeInto(RawSurfaceAttachedState *target) = 0;
+};
+
+/**
+ * The SurfaceAttachedState type represents (typed) state of a surface extension. The main
+ * difference between RawSurfaceAttachedState and SurfaceAttachedState is that the latter is
+ * a convenience helper that does not require implementing the mergeInto() function if the
+ * managed state can be moved using std::move().
+ */
+template<typename Self>
+class SurfaceAttachedState : public RawSurfaceAttachedState
+{
+public:
+    void mergeInto(RawSurfaceAttachedState *target) override
+    {
+        auto self = static_cast<Self *>(this);
+        auto other = static_cast<Self *>(target);
+
+        *other = std::exchange(*self, Self{});
+    }
+};
+
+/**
+ * The RawSurfaceExtension type represents a surface extension whose state must be synchronized
+ * with the double buffered state of the surface.
+ */
+class RawSurfaceExtension
+{
+public:
+    virtual ~RawSurfaceExtension()
+    {
+    }
+
+    /**
+     * Creates the state associated with the extension.
+     */
+    virtual std::unique_ptr<RawSurfaceAttachedState> createState() = 0;
+
+    /**
+     * Applies the state associated with the extension. It is called by the SurfaceInterface
+     * when it applies new surface state.
+     */
+    virtual void applyState(RawSurfaceAttachedState *state) = 0;
+};
+
+/**
+ * The SurfaceExtension type represents a surface extension whose state must be synchronized
+ * with the double buffered state of the surface. The main difference between the
+ * RawSurfaceExtension and the SurfaceExtension is that the latter is a convenience helper that
+ * removes the need for state static casts and registering/unregistering the extension manually.
+ *
+ * The surface extension must provide a `void apply(State *state)` function in order to apply
+ * its state when a new surface state is applied.
+ */
+template<typename Self, typename State>
+class SurfaceExtension : public RawSurfaceExtension
 {
 public:
     explicit SurfaceExtension(SurfaceInterface *surface)
+        : surface(surface)
     {
-        connect(surface, &SurfaceInterface::stateStashed, this, &SurfaceExtension::stashState);
-        connect(surface, &SurfaceInterface::stateApplied, this, &SurfaceExtension::applyState);
+        pending = static_cast<State *>(surface->addExtension(this));
     }
 
-    virtual void apply(Commit *commit) = 0;
+    ~SurfaceExtension() override
+    {
+        if (surface) {
+            surface->removeExtension(this);
+        }
+    }
 
-    Commit pending;
-    QMap<quint32, Commit> stashed;
+    /// The surface associated with the extension.
+    QPointer<SurfaceInterface> surface;
+
+    /// The pending state associated with the extension.
+    State *pending;
 
 private:
-    void stashState(quint32 serial)
+    std::unique_ptr<RawSurfaceAttachedState> createState() override final
     {
-        Commit stash = std::exchange(pending, Commit{});
-        stashed.insert(serial, stash);
+        return std::make_unique<State>();
     }
 
-    void applyState(quint32 serial)
+    void applyState(RawSurfaceAttachedState *state) override final
     {
-        if (!stashed.isEmpty()) {
-            if (stashed.firstKey() == serial) {
-                Commit stash = stashed.take(serial);
-                apply(&stash);
-            }
-            return;
-        }
-
-        apply(&pending);
-        pending = Commit{};
+        static_cast<Self *>(this)->apply(static_cast<State *>(state));
     }
 };
 
